"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async = require("async");
const chalk_1 = require("chalk");
const fs = require("fs");
const path = require("path");
// import statusBoard from 'status-board';
const _ = require("underscore");
const helpers_1 = require("../helpers");
const item_manager_1 = require("../item-manager");
const package_dependency_manager_1 = require("../package-dependency-manager");
const scaffolding_1 = require("./scaffolding");
function directoryHasAtlasBoardProject(dir) {
    // The project should have these items
    const requiredItems = ['packages', 'package.json', 'config'];
    return requiredItems.every((item) => {
        return fs.existsSync(path.join(dir, item));
    });
}
const validNewDirectoryExp = /^[a-zA-Z0-9_-]*$/;
// tslint:disable-next-line max-line-length
function generate(projectDir, packages, itemType, itemName, logger, callback) {
    if (!helpers_1.areValidPathElements([itemType, itemName])) {
        return callback('invalid input');
    }
    const templateFolder = path.join(__dirname, '../..', 'templates', 'new-components');
    // Assert valid parameter usage
    const itemsToGenerate = ['widget', 'dashboard', 'job'];
    if (itemsToGenerate.indexOf(itemType) === -1) {
        // tslint:disable-next-line max-line-length
        return callback('Invalid generator ' + itemType + '\nUse one of: ' + itemsToGenerate.join(', '));
    }
    // Assert a project already exists here
    if (!directoryHasAtlasBoardProject(projectDir)) {
        // tslint:disable-next-line max-line-length
        return callback('It seems that no project exists here yet. Please navigate to your project\'s root directory, or generate one first.');
    }
    // Assert name given
    if (!itemName) {
        // tslint:disable-next-line max-line-length
        return callback('ERROR: No ' + itemType + ' name provided. Please try again with a name after the generate parameter');
    }
    // Assert no such item exists there yet
    const destPackageLocation = path.join(projectDir, 'packages', packages);
    let options = {};
    let target;
    let src;
    if (itemType === 'dashboard') {
        src = path.join(templateFolder, 'dashboard', 'default.json');
        target = path.join(destPackageLocation, 'dashboards', itemName + '.json');
    }
    else {
        src = path.join(templateFolder, itemType);
        target = path.join(path.join(destPackageLocation, itemType + 's'), itemName);
        options = {
            data: {
                name: itemName,
            },
            engine: 'ejs',
            replace: {
                'default.js': itemName + '.js',
                'widget.': itemName + '.',
            },
        };
    }
    if (fs.existsSync(target)) {
        return callback('ERROR: This ' + itemType + ' already seems to exist at ' + target);
    }
    logger.log('\nCreating new %s at %s...', itemType, target);
    scaffolding_1.scaffold(src, target, options, callback);
    logger.log(chalk_1.default.green('SUCCESS !!') + '\n');
}
exports.generate = generate;
function newProject(srcDir, destDir, logger, callback) {
    // Check for valid directory name
    const dirName = path.basename(destDir);
    if (!dirName.match(validNewDirectoryExp)) {
        return callback('Invalid wallboard name');
    }
    if (!helpers_1.isPathContainedInRoot(destDir, process.cwd())) {
        return callback('invalid directory');
    }
    logger.log('\n  Generating a new AtlasBoard project at %s...', destDir.gray);
    const parentDir = path.dirname(destDir);
    if (directoryHasAtlasBoardProject(parentDir)) {
        // tslint:disable-next-line max-line-length
        return callback('You can not create an atlasboard inside a directory containing an atlasboard (at least we think you shouldn\'t)');
    }
    if (fs.existsSync(destDir)) {
        // tslint:disable-next-line max-line-length
        return callback('There is already a directory here called ' + destDir + '. Please choose a new name.');
    }
    logger.log('  Creating new wallboard ...');
    const options = {
        data: {
            name: dirName,
        },
        engine: 'ejs',
    };
    scaffolding_1.scaffold(srcDir, destDir, options, callback);
}
exports.newProject = newProject;
function list(packagesPath, logger, callback) {
    const packagePath = Array.isArray(packagesPath) ? packagesPath : [packagesPath];
    async.map(_.unique(packagePath), (pkgPath, cb) => {
        const packageList = { package: pkgPath };
        item_manager_1.getByPackage(pkgPath, 'widgets', '.js', (err, packagesWidgetList) => {
            if (err) {
                return cb(err);
            }
            packageList.widgets = packagesWidgetList;
            item_manager_1.getByPackage(pkgPath, 'jobs', '.js', (error, packagesJobList) => {
                if (error) {
                    return cb(error);
                }
                packageList.jobs = packagesJobList;
                cb(null, (packageList.widgets.length && packageList.jobs.length) ? packageList : null);
            });
        });
    }, (err, results) => {
        callback(err, _.compact(results));
    });
}
exports.list = list;
function start(options, logger, callback) {
    if (!directoryHasAtlasBoardProject(process.cwd())) {
        // tslint:disable-next-line max-line-length
        return callback('I couldn\'t find a valid AtlasBoard dashboard. Try generating one with `atlasboard new DASHBOARDNAME`.');
    }
    // start Status Board
    // statusBoard(options, callback);
}
exports.start = start;
function install(options, logger, callback) {
    const packagesLocalFolder = path.join(process.cwd(), '/packages');
    package_dependency_manager_1.installDependencies([packagesLocalFolder], callback);
}
exports.install = install;
//# sourceMappingURL=commands-logic.js.map