"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const path = require("path");
const sanitize = require("sanitize-filename");
const commands_logic_1 = require("./commands-logic");
const DEFAULT_PACKAGE_FOLDER = 'default';
/**
 * Generates an Atlasboard component
 *
 * @params args[0] item type: job, dashboard or widget
 * @params args[1] item name
 */
function generate(args, options, logger, callback) {
    if (args.length < 2) {
        callback('Missing arguments. Please use "atlasboard generate widget <mywidget>"');
        return;
    }
    let packageFolder = DEFAULT_PACKAGE_FOLDER;
    const itemType = args[0];
    let itemName = args[1];
    if (itemName.indexOf('#') > -1) {
        // Package namespacing
        packageFolder = itemName.split('#')[0];
        itemName = itemName.split('#')[1];
    }
    return commands_logic_1.generate(process.cwd(), packageFolder, itemType, itemName, logger, (error) => {
        callback(error);
    });
}
exports.generate = generate;
/**
 * Creates a new dashboard
 *
 * @params args[0] dashboard directory
 */
function newProject(args, options, logger, callback) {
    if (args.length < 1) {
        callback('Missing arguments. Please use "atlasboard new <mywallboard>"');
        return;
    }
    const newDirectory = sanitize(args[0]);
    const srcDir = path.join(__dirname, '../..', 'templates', 'new-components', 'project');
    const destDir = path.join(process.cwd(), newDirectory);
    return commands_logic_1.newProject(srcDir, destDir, logger, (error) => {
        if (error) {
            callback(error);
            return;
        }
        process.chdir(newDirectory);
        const isWindows = /^win/.test(process.platform);
        const npmCommand = isWindows ? 'npm.cmd' : 'npm';
        const childProcess = require('child_process');
        const child = childProcess.spawn(npmCommand, ['install', '--production'], { stdio: 'inherit' });
        logger.log('  Installing npm dependencies...');
        child.on('error', () => {
            // tslint:disable-next-line max-line-length
            logger.log('\n  Error installing dependencies. Please run "npm install" inside the dashboard directory');
            callback('Error installing dependencies');
        });
        child.on('exit', () => {
            logger.log([
                chalk_1.default.green('\n  SUCCESS !!') + '\n',
                '  New project "' + chalk_1.default.yellow(newDirectory) + '" successfully created. Now you can:\n',
                chalk_1.default.gray('   1. cd ' + newDirectory),
                chalk_1.default.gray('   2. npm install'),
                chalk_1.default.gray('   3. start your server with `atlasboard start` (or `node start.js`)'),
                chalk_1.default.gray('   4. browse http://localhost:3000') + '\n',
                '   Optionally: import the Atlassian package (or any other package) by running:\n',
                chalk_1.default.gray('   git init'),
                // tslint:disable-next-line max-line-length
                chalk_1.default.gray('   git submodule add https://bitbucket.org/atlassian/atlasboard-atlassian-package packages/atlassian\n'),
            ].join('\n'));
            callback();
        });
    });
}
exports.newProject = newProject;
/**
 * List of all Atlasboard components (widgets or jobs) within all available packages
 */
function list(args, options, logger, callback) {
    function parse(packages) {
        logger.log('        Package "' + path.basename(packages.dir) + '":');
        packages.items.forEach((item) => {
            logger.log('          - ' + chalk_1.default.gray(path.basename(item, '.js')));
        });
    }
    const packagesLocalFolder = path.join(process.cwd(), '/packages');
    const packagesAtlasboardFolder = path.join(__dirname, '../../packages');
    const packagesPath = [packagesLocalFolder, packagesAtlasboardFolder];
    return commands_logic_1.list(packagesPath, logger, (err, packages) => {
        if (err) {
            callback('Error reading package folder');
            return;
        }
        packages.forEach((packagez) => {
            logger.log('Path: ' + packagez.package + ':\n');
            logger.log(chalk_1.default.yellow('  - Widgets:'));
            packagez.widgets.forEach(parse);
            logger.log(chalk_1.default.yellow('  - Jobs:'));
            packagez.jobs.forEach(parse);
        });
        logger.log('');
        callback();
    });
}
exports.list = list;
/**
 * When run in a project's base directory, starts the AtlasBoard server.
 *
 * @params args[0] port (optional)
 * @params args --jobFilter filter by job (optional)
 * @params args --dashboardFilter filter by dashboard (optional)
 */
function start(args, options, logger, callback) {
    const port = isNaN(args[0]) ? 3000 : args[0];
    const statusBoardOptions = { port, filters: {}, install: true };
    const argsOptimistic = require('optimist')(args).argv;
    if (argsOptimistic.noinstall) {
        statusBoardOptions.install = false;
    }
    if (argsOptimistic.job) {
        logger.log('Loading jobs matching ' + chalk_1.default.yellow(argsOptimistic.job) + ' only');
        statusBoardOptions.filters.jobFilter = argsOptimistic.job;
    }
    if (argsOptimistic.dashboard) {
        logger.log('Loading dashboards matching ' + chalk_1.default.yellow(argsOptimistic.dashboard) + ' only');
        statusBoardOptions.filters.dashboardFilter = argsOptimistic.dashboard;
    }
    logger.log(chalk_1.default.gray('\nStarting server...'));
    commands_logic_1.start(statusBoardOptions, logger, (err) => {
        callback(err);
    });
}
exports.start = start;
/**
 * Install the packages dependencies
 */
function install(args, options, logger, callback) {
    // check for the right arguments
    return commands_logic_1.install({}, logger, (err) => {
        callback(err);
    });
}
exports.install = install;
//# sourceMappingURL=commands.js.map