"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async = require("async");
const fs = require("fs");
const path = require("path");
const _ = require("underscore");
const logger_1 = require("./logger");
/**
 * Filters will be applied to a certain type to decide if that
 * item should be included in the list to be returned.
 * You can match extensions, do sanity checks (valid JSON), etc.
 */
const filters = {
    dashboards(dashboardPath) {
        try {
            const contentJSON = JSON.parse(fs.readFileSync(dashboardPath).toString());
            return (contentJSON.enabled !== false);
        }
        catch (e) {
            // tslint:disable-next-line max-line-length
            logger_1.default().error('## ERROR ## ' + dashboardPath + ' has an invalid format or file doesn\'t exist\n');
            return false;
        }
    },
};
/**
 * Returns relative path to packages path based on item type.
 * The main purpose of this function is to be able to handle the different way
 * widgets and jobs paths are resolved compared to the way dashboards paths are.
 *
 * @param {string} name item name to match. i.e: widget1
 * @param {string} itemType item type in plural. ('dashboards', 'jobs', 'widgets')
 * @param {string} extension : file extension
 *
 * @return {string} relative path to item: jobs/job1/job1.js
 */
function resolveLocation(name, itemType, extension) {
    const useDirectoryLevel = ((itemType === 'widgets') || (itemType === 'jobs'));
    if (useDirectoryLevel) {
        // jobs/job1/job1.js
        return path.join(itemType, name, name + extension);
    }
    // dashboards/dashboard.json
    return path.join(itemType, name + extension);
}
exports.resolveLocation = resolveLocation;
/**
 * Get the items that match the particular filter.
 *
 * @param {[string]} items : list of file paths
 * @param {string} name item name to match. It can be namespaced. i.e: atlassian#widget1, widget1
 * @param {string} itemType item type in plural. ('dashboards', 'jobs', 'widgets')
 * @param {string} extension : filter result by extension
 */
function resolveCandidates(items, name, itemType, extension) {
    let searchCriteria = '';
    if (name.indexOf('#') > -1) {
        const packageName = name.split('#')[0];
        const itemParsedName = name.split('#')[1];
        // package/jobs/job1/job1.js
        // tslint:disable-next-line max-line-length
        searchCriteria = path.join(packageName, this.resolveLocation(itemParsedName, itemType, extension));
    }
    else {
        // jobs/job1/job1.js
        searchCriteria = this.resolveLocation(name, itemType, extension);
    }
    searchCriteria = path.sep + searchCriteria;
    return items.filter((item) => {
        return item.indexOf(searchCriteria) > -1;
    });
}
exports.resolveCandidates = resolveCandidates;
/**
 * Return first candidate found matching name, type and extension
 *
 * @param {[string]} packagesPath : list of directories to find packages in.
 * @param {string} itemName item name to match.
 * It can be namespaced. i.e: atlassian#widget1, widget1
 * @param {string} itemType item type in plural. ('dashboards', 'jobs', 'widgets')
 * @param {string} extension : filter result by extension
 * @param {function} callback : callback
 */
// tslint:disable-next-line max-line-length
function getFirst(packagesPath, itemName, itemType, extension, callback) {
    this.get(packagesPath, itemType, extension, (err, items) => {
        if (err) {
            return callback(err);
        }
        const candidates = this.resolveCandidates(items, itemName, itemType, extension);
        callback(null, candidates.length ? candidates[0] : null);
    });
}
exports.getFirst = getFirst;
/**
 * Return list of items found in any package within packagesPath
 *
 * @param {[string]} packagesPath : list of directories to find packages in.
 * @param {string} itemType item type in plural. ('dashboards', 'jobs', 'widgets')
 * @param {string} extension : filter result by extension
 * @param {function} callback : callback
 */
function get(packagesPath, itemType, extension, callback) {
    this.getByPackage(packagesPath, itemType, extension, (err, results) => {
        if (err) {
            return callback(err);
        }
        let items = [];
        results.forEach((packages) => {
            items = items.concat(packages.items);
        });
        callback(null, items);
    });
}
exports.get = get;
/**
 * Return list of items found in any package within packagesPath
 * Items are returned separated by package
 *
 * @param {[string]} packagesPath : list of directories to find packages in.
 * @param {string} itemType item type in plural. ('dashboards', 'jobs', 'widgets')
 * @param {string} extension : filter result by extension
 * @param {function} callback : callback
 */
function getByPackage(packagesPath, itemType, extension, callback) {
    let processPackagesPath;
    if (!Array.isArray(packagesPath)) {
        processPackagesPath = [packagesPath];
    }
    else {
        processPackagesPath = packagesPath;
    }
    function readItemsFromPackageDir(dir, cb) {
        const packages = { dir };
        const itemDir = path.join(dir, itemType);
        if (!fs.existsSync(itemDir)) {
            packages.items = [];
            return cb(null, packages);
        }
        // this functions parses:
        // - packages/default/<itemType>/*
        // - packages/otherpackages/<itemType>/*
        // for dashboards, or:
        // - packages/default/<itemType>/*/*.js
        // - packages/otherpackages/<itemType>/*/*.js
        // for jobs and widgets
        fs.readdir(itemDir, (err, items) => {
            if (err) {
                return cb(err);
            }
            let selectedItems = [];
            items.forEach((itemName) => {
                let item = path.join(itemDir, itemName);
                const stat = fs.statSync(item);
                if (stat.isDirectory()) {
                    // /job/job1/job1.js
                    item = path.join(item, itemName + extension);
                }
                if (path.extname(item) === extension) {
                    if (fs.existsSync(item)) {
                        selectedItems.push(item);
                    }
                }
            });
            if (filters[itemType]) {
                selectedItems = selectedItems.filter(filters[itemType]);
            }
            packages.items = selectedItems;
            return cb(null, packages);
        });
    }
    // this function read all the packages from the provided directory packagesPath:
    // - packages/default/*
    // - packages/otherpackages/*
    // and calls readItemsFromPackageDir for every one of them
    function fillPackages(packagePath, cb) {
        fs.readdir(packagePath, (error, allPackagesDir) => {
            if (error) {
                return cb(error);
            }
            let processedAllPackagesDir;
            // convert to absolute path
            processedAllPackagesDir = allPackagesDir.map((partialDir) => {
                return path.join(packagePath, partialDir);
            });
            // get only valid directories
            processedAllPackagesDir = processedAllPackagesDir.filter((dir) => {
                return fs.statSync(dir).isDirectory();
            });
            // read items from every package and flatten results
            async.map(processedAllPackagesDir, readItemsFromPackageDir, (err, results) => {
                if (err) {
                    return cb(err);
                }
                cb(null, _.flatten(results));
            });
        });
    }
    // process all package paths
    async.map(processPackagesPath.filter(fs.existsSync), fillPackages, (err, results) => {
        if (err) {
            return callback(err);
        }
        callback(null, _.flatten(results));
    });
}
exports.getByPackage = getByPackage;
//# sourceMappingURL=item-manager.js.map