"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async = require("async");
const fs = require("fs");
const path = require("path");
const readJson = require("read-package-json");
const semver = require("semver");
const _ = require("underscore");
function installDependencies(packagesPath, callback) {
    // Process all available package containers
    // tslint:disable-next-line max-line-length
    async.map(packagesPath.filter(fs.existsSync), checkPackagesFolder, (mapError, results) => {
        if (mapError) {
            return callback(mapError);
        }
        const paths = _.flatten(results);
        async.eachSeries(paths, checkValidIfAtlasboardVersionForPackage, (eachSeriesError) => {
            if (eachSeriesError) {
                return callback(eachSeriesError);
            }
            async.eachSeries(paths, install, (error) => {
                callback(error);
            });
        });
    });
}
exports.installDependencies = installDependencies;
// in both test and production env will be located here.
const atlasboardPackageJsonPath = path.join(__dirname, '../');
/**
 * Search for packages in the current folder
 */
function checkPackagesFolder(packagesPath, cb) {
    fs.readdir(packagesPath, (err, allPackagesDir) => {
        if (err) {
            return cb(err);
        }
        let processedPackagesDir;
        // convert to absolute path
        processedPackagesDir = allPackagesDir.map((partialDir) => {
            return path.join(packagesPath, partialDir);
        });
        // make sure we have package.json file
        processedPackagesDir = processedPackagesDir.filter((dir) => {
            return fs.statSync(dir).isDirectory() && fs.existsSync(dir + '/package.json');
        });
        cb(null, processedPackagesDir);
    });
}
/**
 * Install from package folder
 */
function getValidPackageJSON(pathPackage, callback) {
    readJson(pathPackage + '/package.json', callback);
}
/**
 * Install from package folder
 */
function checkValidIfAtlasboardVersionForPackage(pathPackage, callback) {
    getValidPackageJSON(pathPackage, (err, packageJson) => {
        if (err) {
            return callback(err);
        }
        getValidPackageJSON(atlasboardPackageJsonPath, (error, atlasboardPackageJson) => {
            if (error) {
                return callback('package.json not found for atlasboard at ' + atlasboardPackageJsonPath);
            }
            if (packageJson.engines && packageJson.engines.atlasboard) {
                const ok = semver.satisfies(atlasboardPackageJson.version, packageJson.engines.atlasboard);
                // tslint:disable-next-line max-line-length
                const msg = 'Atlasboard version does not satisfy package dependencies at ' + pathPackage + '. Please consider updating your version of atlasboard. Version required: ' + packageJson.engines.atlasboard + '. Atlasboard version: ' + atlasboardPackageJson.version;
                callback(ok ? null : msg);
            }
            else {
                // not atlasboard reference in engines node
                callback(null);
            }
        });
    });
}
/**
 * Install from package folder
 */
function install(pathPackageJson, callback) {
    // save current path
    const currPath = process.cwd();
    process.chdir(pathPackageJson);
    const isWindows = /^win/.test(process.platform);
    const npmCommand = isWindows ? 'npm.cmd' : 'npm';
    // tslint:disable-next-line max-line-length
    executeCommand(npmCommand, ['install', '--production', pathPackageJson], (err, code) => {
        if (err) {
            callback('Error installing dependencies for ' + pathPackageJson + '. err:' + err);
        }
        else {
            callback(code !== 0 ? 'error installing ' + pathPackageJson : null);
        }
    });
    // Restore path
    process.chdir(currPath);
}
/**
 * Executes a command in a childProcess
 */
function executeCommand(cmd, args, callback) {
    const childProcess = require('child_process');
    const child = childProcess.spawn(cmd, args, { stdio: 'inherit' });
    child.on('error', (err) => {
        callback(err);
    });
    child.on('exit', (code) => {
        callback(null, code);
    });
}
//# sourceMappingURL=package-dependency-manager.js.map